
//Title:        Emulador para o Kit MC8051
//Version:
//Copyright:    Copyright (c) 1997
//Author:       Hermes F. Reis Neto / Jan L. de Lara / Kleber W. Tolezani
//Company:      Escola Politcnica da Universidade de So Paulo
//Description:


package MC8051;

import java.io.*;
import java.awt.*;

public class Console {
  public static boolean tecl_disp = false;

  public static void mostraRAMInt(int endini, int endfim) {
    int endRAM = endini;
    int i = 0;

    while(endRAM <= endfim) {
      if((i % 8) == 0)
        System.out.print("\n0x" + Integer.toHexString(endRAM) + ":");

      System.out.print("\t0x" + Integer.toHexString(Memoria.memint[endRAM]));

      endRAM++;
      i++;
    }

    System.out.println();
    System.out.println();
  }

  public static String aguardaTecla() {
    String str = null;
    try {
      DataInputStream in = new DataInputStream(System.in);

      System.out.print("> ");
      str = in.readLine();
    }
    catch(Exception e) {
      System.out.println("ERRO: " + e);
      str = "";
    }

    return str;
  }

  public static void imprimeHelp() {
    System.out.println();
    System.out.println("Comandos validos: g <endereco_hex>");
    System.out.println("                  g +<num. instr.>");
    System.out.println("                  pc <endereco_hex>");
    System.out.println("                  pc +<num. instr.>");
    System.out.println("                  ch <enderecoRAM_hex_2dig> <valor_hex_2dig>");
    System.out.println("                  sh <enderecoRAM_inicio_hex_2dig> <enderecoRAM_fim_hex_2dig>");
    System.out.println("                  l <nome_arq_HEX>");
    System.out.println("                  p");
    System.out.println("                  h");
    System.out.println("                  q");
    System.out.println();
  }

  public static void executa () {
    Emulador em = null;
    String cmd = null;
    boolean cmdOK = false;
    int endereco = -1;
    int numinstr = 1;
    try {
      if(tecl_disp)
        em = new Emulador(true,true);
      else
        em = new Emulador(false,false);
    }
    catch(Exception e) {
      System.out.println("\n\nERRO: Excecao: " + e);
      System.exit(-1);
    }

    while(true) {
      System.out.println();
      Memoria.imprimeEstado();
      System.out.println();

      endereco = -1;
      numinstr = 1;
      cmdOK = false;
      while(!cmdOK) {
        cmd = aguardaTecla();
        if(cmd.startsWith("l ")) {
          int erro = Memoria.leArquivoHEX(cmd.substring(2));
          if(erro != 0) {
            System.out.println("leArquivoHEX() retornou " + erro);
            aguardaTecla();
            System.exit(-1);
          }
          Memoria.PC = 0x8000;
        }
        else if(cmd.startsWith("g ")) {
          if(cmd.charAt(2) == '+') {
            try {
              numinstr = Integer.parseInt(cmd.substring(3));
              cmdOK = true;
            } catch(NumberFormatException e) {
              System.out.println("ERRO: comando ou parametro invalido!");
              imprimeHelp();
            }
          }
          else if(cmd.charAt(2) != '-'){
            try {
              endereco = Integer.parseInt(cmd.substring(2),16);
              cmdOK = true;
            } catch(NumberFormatException e) {
              System.out.println("ERRO: comando ou parametro invalido!");
              imprimeHelp();
            }
          }
          else {
            System.out.println("ERRO: comando ou parametro invalido!");
            imprimeHelp();
          }
        }
        else if(cmd.startsWith("pc ")) {
          if(cmd.charAt(3) == '+') {
            try {
              numinstr = Integer.parseInt(cmd.substring(4),10);

              endereco = Memoria.PC;
              while(numinstr > 0) {
                endereco = Memoria.PC + Instr.nbytes[Memoria.leROM(endereco)];
                endereco &= 0xffff;
                numinstr--;
              }
              Memoria.PC = endereco;
              endereco = -1;

              cmdOK = true;
            } catch(NumberFormatException e) {
              System.out.println("ERRO: comando ou parametro invalido!");
              imprimeHelp();
            }
          }
          else if(cmd.charAt(3) != '-'){
            try {
              endereco = Integer.parseInt(cmd.substring(3),16);

              Memoria.PC = endereco & 0xffff;
              endereco = -1;
              numinstr = 0;

              cmdOK = true;
            } catch(NumberFormatException e) {
              System.out.println("ERRO: comando ou parametro invalido!");
              imprimeHelp();
            }
          }
          else {
            System.out.println("ERRO: comando ou parametro invalido!");
            imprimeHelp();
          }
        }
        else if(cmd.length()>7 && cmd.startsWith("ch ") && cmd.charAt(5)==' '){
          try {
            int reg = Integer.parseInt(cmd.substring(3,5),16) & 0xff;
            int val = Integer.parseInt(cmd.substring(6),16) & 0xff;

            Memoria.gravaRAMInt(reg, (short)val);
            if(Memoria.TCON_alterado || Memoria.TMOD_alterado)
              em.tms.configura();
            if(Memoria.SCON_alterado)
              em.ser.configura();

            endereco = -1;
            numinstr = 0;

          } catch (NumberFormatException e) {
              System.out.println("ERRO: comando ou parametro invalido!");
              imprimeHelp();
          }
        }
        else if(cmd.length()>7 && cmd.startsWith("sh ") && cmd.charAt(5)==' '){
          try {
            int ini = Integer.parseInt(cmd.substring(3,5),16) & 0xff;
            int fim = Integer.parseInt(cmd.substring(6),16) & 0xff;

            mostraRAMInt(ini, fim);

            endereco = -1;
            numinstr = 0;

          } catch (NumberFormatException e) {
              System.out.println("ERRO: comando ou parametro invalido!");
              imprimeHelp();
          }
        }
        else if(cmd.equals("q")) {
          System.out.println();
          System.out.println("Encerrando programa.");
          System.out.println();
          System.exit(0);
        }
        else if(cmd.equals("h")) {
          imprimeHelp();
        }
        else if(cmd.equals("p")) {
          endereco = -1;
          numinstr = 0;
          cmdOK = true;
        }
        else if(!cmd.equals("")) {
          System.out.println("ERRO: comando nao reconhecido!");
          imprimeHelp();
        }
        else {
          numinstr = 1;
          endereco = -1;
          cmdOK = true;
        }
      }

      if(endereco != -1) {
        boolean old_breakpoint = Memoria.breakpoint[endereco];
        Memoria.breakpoint[endereco] = true;
        em.executa(-1);
        Memoria.breakpoint[endereco] = old_breakpoint;
      }
      else
        if(numinstr > 0)
          em.executa(numinstr);
    }
  }

  static boolean packFrame = false;

  public static void main(String[] args) {
    if(args.length==1 && args[0].equals("-tecl_disp"))
      tecl_disp = true;
    else if(args.length != 0)
      System.out.println("\n\nAVISO: parametro(s) ignorado(s).\nUtilizacao:  "
       + "MC8051.Console  [-tecl_disp]\n");

    executa();
  }
}
