
//Title:        Emulador para o Kit MC8051
//Version:
//Copyright:    Copyright (c) 1997
//Author:       Hermes F. Reis Neto / Jan L. de Lara / Kleber W. Tolezani
//Company:      Escola Politcnica da Universidade de So Paulo
//Description:


package MC8051;

public class Emulador {
  public static final String nomeClasseDisplay = "MC8051.Display";
  public static final int faixaEnderecoDisplay = 0;

  public ProcInstr pr;
  public Timers tms;
  public Serial ser;
  public GerenciadorES gerES;
  public boolean mostrarTelas;

  public Emulador(boolean mostrarTelas, boolean usarES)
    throws ClassNotFoundException, InstantiationException, ClassCastException,
    IllegalAccessException
  {
    if(!Instr.pronto)
      Instr.inicializa();
    Memoria.inicializa(true);

    this.mostrarTelas = mostrarTelas;

    pr = new ProcInstr();
    tms = new Timers();
    ser = new Serial();

    gerES = new GerenciadorES(mostrarTelas);
    gerES.ativo = usarES;
    gerES.adicionaTeclado(new Teclado());
    gerES.adicionaModuloES(nomeClasseDisplay, faixaEnderecoDisplay);

    Memoria.es = gerES;

    reset();
  }

  public void reset() {
    Memoria.inicializa(false);

    tms.reset();
    ser.reset();
    gerES.reset();
  }

  public void executa(int numInstr) {
    boolean parar = false;
    int nciclos = 0;

    while(!parar) {
      // EXECUTA A INSTRUO ATUAL E PEGA O NMERO DE CICLOS DE RELGIO DE
      // DURAO DA MESMA
      nciclos = pr.processa();

      // ATUALIZA TEMPO NOS COMPONENTES DEPENDENTES DO TEMPO.
      tms.computaCiclos(nciclos);
      ser.computaCiclos(nciclos);
      if(gerES.ativo)
        gerES.computaCiclos(nciclos);

      // ATUALIZA TELAS SE ESTIVER EM MODO GRFICO
      if(mostrarTelas && gerES.ativo)
        gerES.atualizaTela();

      parar = Memoria.breakpoint[Memoria.PC];
      if(!parar && numInstr > 0) {
        numInstr--;
        parar = numInstr == 0;
      }
    }
  }

  public void executaUma() {
    int nciclos = 0;

    // EXECUTA A INSTRUO ATUAL E PEGA O NMERO DE CICLOS DE RELGIO DE
    // DURAO DA MESMA
    nciclos = pr.processa();

    // ATUALIZA TEMPO NOS COMPONENTES DEPENDENTES DO TEMPO.
    tms.computaCiclos(nciclos);
    ser.computaCiclos(nciclos);
    if(gerES.ativo)
      gerES.computaCiclos(nciclos);

    // ATUALIZA TELAS SE ESTIVER EM MODO GRFICO
    if(mostrarTelas && gerES.ativo)
      gerES.atualizaTela();
  }
}
