
//Title:        Emulador para o Kit MC8051
//Version:
//Copyright:    Copyright (c) 1997
//Author:       Hermes F. Reis Neto / Jan L. de Lara / Kleber W. Tolezani
//Company:      Escola Politcnica da Universidade de So Paulo
//Description:


package MC8051;

import java.util.*;

/*
- Faixas de endereo para os mdulos de E/S, pelo nmero do mdulo:

  0 (CS0) - 2000 a 20ff, 2800 a 28ff, 3000 a 30ff, 3800 a 38ff
  1 (CS1) - 2100 a 21ff, 2900 a 29ff, 3100 a 31ff, 3900 a 39ff
  2 (CS2) - 2200 a 22ff, 2a00 a 2aff, 3200 a 32ff, 3a00 a 3aff
  3 (CS3) - 2300 a 23ff, 2b00 a 2bff, 3300 a 33ff, 30b0 a 3bff
  4 (CS4) - 2400 a 24ff, 2c00 a 2cff, 3400 a 34ff, 3c00 a 3cff
  5 (CS5) - 2500 a 25ff, 2d00 a 2dff, 3500 a 35ff, 3d00 a 3dff
  6 (CS6) - 2600 a 26ff, 2e00 a 2eff, 3600 a 36ff, 3e00 a 3eff
  7 (CS7) - 2700 a 27ff, 2f00 a 2fff, 3700 a 37ff, 3f00 a 3fff
  8 - 0000 a 1fff, 4000 a 7fff
*/

public class GerenciadorES {
  public boolean ativo;
  public boolean mostrarTelas;
  public Teclado tcl;
  public ModuloES[] modulos = new ModuloES[9];

  public GerenciadorES(boolean mostrarTelas) {
    this.mostrarTelas = mostrarTelas;
    ativo = true;
    tcl = null;
    for(int m=0; m<9; m++)
      modulos[m] = null;
  }

  public boolean adicionaModuloES(String nomeClasse, int faixaEnd)
    throws ClassNotFoundException, InstantiationException, ClassCastException,
    IllegalAccessException
  {
    ModuloES mdl;

    if(faixaEnd < 0 || faixaEnd > 8)
      return false;

    mdl = (ModuloES) Class.forName(nomeClasse).newInstance();

    if(modulos[faixaEnd] != null)
      return false;

    mdl.ativo = true;
    if(mostrarTelas) {
      mdl.criaTela();
      mdl.mostraTela();
    }
    modulos[faixaEnd] = mdl;

    return true;
  }

  public ModuloES removeModuloES(int faixaEnd) {
    if(faixaEnd < 0 || faixaEnd > 8)
      return null;
    modulos[faixaEnd]=null;
    return modulos[faixaEnd];
  }

  public ModuloES encontraModuloES(int endereco) {
    if((endereco & 0xe000) == 0x2000)
      return modulos[(endereco & 0x0700) >> 8];
    else
      return modulos[8];
  }

  public int leDado(int enderecoES) {
    ModuloES mdl = encontraModuloES(enderecoES);
    if(mdl != null && mdl.ativo)
      return mdl.leDado(enderecoES);
    else
      return 0xff;
  }

  public void escreveDado(int enderecoES, short dado) {
//    System.out.println("gerenciadorES: escrita em " + Integer.toHexString(enderecoES));
    ModuloES mdl = encontraModuloES(enderecoES);
    if(mdl != null && mdl.ativo)
      mdl.escreveDado(enderecoES, dado);
/*    if(mdl != null)
      System.out.println("gerenciadorES: modulo " + mdl.nome());
    else
      System.out.println("gerenciadorES: nao encontrei modulo!");
*/

  }

  public boolean adicionaTeclado(Teclado tcl) {
    if(this.tcl != null)
      return false;

    if(mostrarTelas) {
      tcl.criaTela();
      tcl.mostraTela();
    }

    this.tcl = tcl;
    return true;
  }

  public boolean configuraTeclado(int nciclos_pressionamento,
    int nciclos_bounce, int nciclos_estado_bounce) {
    if(tcl == null)
      return false;

    if(nciclos_estado_bounce > nciclos_bounce ||
      nciclos_bounce > nciclos_pressionamento
    )
      return false;

    tcl.nciclos_pressionamento = nciclos_pressionamento;
    tcl.nciclos_bounce = nciclos_bounce;
    tcl.nciclos_estado_bounce = nciclos_estado_bounce;

    return true;
  }

  public int leTeclado() {
    if(tcl == null)
      return Memoria.memint[Memoria.P1];
    else
      return tcl.computaValorP1();
  }

  public void computaCiclos(int nciclos) {
    for(int m=0; m<9; m++) {
      if(modulos[m] != null && modulos[m].ativo)
        modulos[m].computaCiclos(nciclos);
    }

    if(tcl != null && tcl.ativo)
      tcl.computaCiclos(nciclos);
  }

  public void atualizaTela() {
    if(!mostrarTelas)
      return;

    for(int m=0; m<9; m++) {
      if(modulos[m] != null && modulos[m].ativo)
        modulos[m].atualizaTela();
    }

    if(tcl != null && tcl.ativo)
      tcl.atualizaTela();
  }

  public void reset() {
    for(int m=0; m<9; m++) {
      if(modulos[m] != null && modulos[m].ativo)
        modulos[m].reset();
    }

    if(tcl != null && tcl.ativo)
      tcl.reset();
  }
}