
//Title:        Emulador para o Kit MC8051
//Version:
//Copyright:    Copyright (c) 1997
//Author:       Hermes F. Reis Neto / Jan L. de Lara / Kleber W. Tolezani
//Company:      Escola Politcnica da Universidade de So Paulo
//Description:


package MC8051;

public class Instr {

  public static final int            SEMPAR =  0; // ex. RETI
  public static final int               ACC =  1; // ex. INC A
  public static final int               REG =  2; // ex. INC R2
  public static final int            INDREG =  3; // ex. INC @R0
  public static final int              END8 =  4; // ex. SJMP 0xf0
  public static final int             END11 =  5; // ex. AJMP 0x700
  public static final int             END16 =  6; // ex. LJMP 0x1000
  public static final int               BIT =  7; // ex. CLR ACC.0
  public static final int             CARRY =  8; // ex. CLR C
  public static final int              DPTR =  9; // ex. INC DPTR
  public static final int               RES = 10; // instruo 0xa5
  public static final int           ACC_REG = 11; // ex. MOV A, R2
  public static final int        ACC_INDREG = 12; // ex. MOVX A, @R0
  public static final int             ACC_B = 13; // ex. DIV AB
  public static final int      ACC_ENDDADOS = 14; // ex. ADD A, B
  public static final int ACC_ENDDADOS_END8 = 15; // ex. CJNE A, EA, 0xc8
  public static final int           ACC_IMM = 16; // ex. MOV A, #0x22
  public static final int      ACC_IMM_END8 = 17; // ex. CJNE A, #0x20, 0xc8
  public static final int       ACC_INDDPTR = 18; // ex. MOVX A, @DPTR
  public static final int ACC_ACC_MAIS_DPTR = 19; // ex. MOVC A, @A + DPTR
  public static final int   ACC_ACC_MAIS_PC = 20; // ex. MOVC A, @A + PC
  public static final int     ACC_MAIS_DPTR = 21; // ex. JMP @A + DPTR
  public static final int           REG_ACC = 22; // ex. MOV R3, A
  public static final int      REG_ENDDADOS = 23; // ex. MOV R2, TCON
  public static final int          REG_END8 = 24; // ex. DJNZ R0, 0xf6
  public static final int           REG_IMM = 25; // ex. MOV R1, #0x33
  public static final int      REG_IMM_END8 = 26; // ex. CJNE R5, #0x20, 0xf7
  public static final int        INDREG_ACC = 27; // ex. MOVX @R1, A
  public static final int   INDREG_ENDDADOS = 28; // ex. MOV @R0, 0x11
  public static final int        INDREG_IMM = 29; // ex. MOV @R0, #0x65
  public static final int   INDREG_IMM_END8 = 30; // ex. CJNE @R1, #0x2c, 0xd3
  public static final int ENDDADOS_ENDDADOS = 31; // ex. MOV EA, 0x3d
  public static final int         BIT_CARRY = 32; // ex. MOV EA.2, C
  public static final int          BIT_END8 = 33; // ex. JB P1.0, 0xc2
  public static final int         CARRY_BIT = 34; // ex. MOV C, P2.0
  public static final int    CARRY_COMPLBIT = 35; // ex. ANL C, /ACC.3
  public static final int          ENDDADOS = 36; // ex. PUSH B
  public static final int      ENDDADOS_ACC = 37; // ex. MOV SBUF, A
  public static final int     ENDDADOS_END8 = 38; // ex. DJNZ 0x20, 0xf8
  public static final int      ENDDADOS_IMM = 39; // ex. MOV EA, #0x40
  public static final int   ENDDADOS_INDREG = 40; // ex. MOV TCON, @R0
  public static final int      ENDDADOS_REG = 41; // ex. MOV TCON, R3
  public static final int          DPTR_IMM = 42; // ex. MOV DPTR, #0x7777
  public static final int       INDDPTR_ACC = 43; // ex. MOVX @DPTR, A

  public static String[] mnem = new String[256];
  public static byte[] tipo = new byte[256];
  public static byte[] nbytes = new byte[256];  // redundante !!
  public static byte[] nciclos = new byte[256];
  public static boolean pronto = false;

  public static String BitString(int numbit) {
    if(numbit < 0x80) {
      return "(bit)" + Integer.toHexString(numbit) + "H";
    }
    else {
      int posbit = numbit % 8;
      String pref = null;
      switch(numbit - posbit) {
        case 0x80: { pref = "P0."  ; break; } // 0x80 a 0x87
        case 0x88: { pref = "TCON."; break; } // 0x88 a 0x8f
        case 0x90: { pref = "P1."  ; break; } // 0x90 a 0x97
        case 0x98: { pref = "SCON."; break; } // 0x98 a 0x9f
        case 0xa0: { pref = "P2."  ; break; } // 0xa0 a 0xa7
        case 0xa8: { pref = "IE."  ; break; } // 0xa8 a 0xaf
        case 0xb0: { pref = "P3."  ; break; } // 0xb0 a 0xb7
        case 0xb8: { pref = "IP."  ; break; } // 0xb8 a 0xbf
        case 0xd0: { pref = "PSW." ; break; } // 0xd0 a 0xd7
        case 0xe0: { pref = "ACC." ; break; } // 0xe0 a 0xe7
        case 0xf0: { pref = "B."   ; break; } // 0xf0 a 0xf7
        default:  {
          pref = "(bit)" + Integer.toHexString(numbit - (numbit%8)) + "H.";
        }
      }
      return pref + posbit;
    }
  }

  public static String SFRString(int enddados) {
    if(enddados < 0x80) {
      return "(dado)" + Integer.toHexString(enddados) + "H";
    }
    else {
      switch(enddados) {
        case 0x80: return "P0";
        case 0x81: return "SP";
        case 0x82: return "DPL";
        case 0x83: return "DPH";
        case 0x87: return "PCON";
        case 0x88: return "TCON";
        case 0x89: return "TMOD";
        case 0x8a: return "TL0";
        case 0x8b: return "TL1";
        case 0x8c: return "TH0";
        case 0x8d: return "TH1";
        case 0x90: return "P1";
        case 0x98: return "SCON";
        case 0x99: return "SBUF";
        case 0xa0: return "P2";
        case 0xa8: return "IE";
        case 0xb0: return "P3";
        case 0xb8: return "IP";
        case 0xd0: return "PSW";
        case 0xe0: return "ACC";
        case 0xf0: return "B";
        default: return "(dado)" + Integer.toHexString(enddados) + "H";
      }
    }
  }

  public static String ASMString(int b0, int b1, int b2) {
    String sep = " ";

    switch(tipo[b0]) {
      case SEMPAR: {
        return mnem[b0];
      }
      case ACC: {
        return mnem[b0] + sep + "A";
      }
      case REG: {
        int numreg = b0 % 8;
        return mnem[b0] + sep + "R" + numreg;
      }
      case INDREG: {
        int numreg = b0 % 2;
        return mnem[b0] + sep + "@R" + numreg;
      }
      case END8: {
        return mnem[b0] + sep + Integer.toHexString(b1) + "H";
      }
      case END11: {
        int end11 = ((b0 & 0x0f) << 8) + b1;
        return mnem[b0] + sep + Integer.toHexString(end11) + "H";
      }
      case END16: {
        int end16 = (b1 << 8) + b2;
        return mnem[b0] + sep + Integer.toHexString(end16) + "H";
      }
      case BIT: {
        return mnem[b0] + sep + BitString(b1);
      }
      case CARRY: {
        return mnem[b0] + sep + "C";
      }
      case DPTR: {
        return mnem[b0] + sep + "DPTR";
      }
      case RES: {
        return mnem[b0];
      }
      case ACC_REG: {
        int numreg = b0 % 8;
        return mnem[b0] + sep + "A, R" + numreg;
      }
      case ACC_INDREG: {
        int numreg = b0 % 2;
        return mnem[b0] + sep + "A, @R" + numreg;
      }
      case ACC_B: {
        return mnem[b0] + sep + "AB";
      }
      case ACC_ENDDADOS: {
        return mnem[b0] + sep + "A, " + SFRString(b1);
      }
      case ACC_ENDDADOS_END8: {
        return mnem[b0] + sep + "A, " + SFRString(b1) + ", " +
          Integer.toHexString(b2) + "H";
      }
      case ACC_IMM: {
        return mnem[b0] + sep + "A, #" + Integer.toHexString(b1) + "H";
      }
      case ACC_IMM_END8: {
        return mnem[b0] + sep + "A, #" + Integer.toHexString(b1) +
          "H, " + Integer.toHexString(b2) + "H";
      }
      case ACC_INDDPTR: {
        return mnem[b0] + sep + "A, @DPTR";
      }
      case ACC_ACC_MAIS_DPTR: {
        return mnem[b0] + sep + "A, @A + DPTR";
      }
      case ACC_ACC_MAIS_PC: {
        return mnem[b0] + sep + "A, @A + PC";
      }
      case ACC_MAIS_DPTR: {
        return mnem[b0] + sep + "@A + DPTR";
      }
      case REG_ACC: {
        int numreg = b0 % 8;
        return mnem[b0] + sep + "R" + numreg + ", A";
      }
      case REG_ENDDADOS: {
        int numreg = b0 % 8;
        return mnem[b0] + sep + "R" + numreg + ", " + SFRString(b1);
      }
      case REG_END8: {
        int numreg = b0 % 8;
        return mnem[b0] + sep + "R" + numreg + ", " + Integer.toHexString(b1) + "H";
      }
      case REG_IMM: {
        int numreg = b0 % 8;
        return mnem[b0] + sep + "R" + numreg + ", #" +
          Integer.toHexString(b1) + "H";
      }
      case REG_IMM_END8: {
        int numreg = b0 % 8;
        return mnem[b0] + sep + "R" + numreg + ", #" +
          Integer.toHexString(b1) + "H, " + Integer.toHexString(b2) + "H";
      }
      case INDREG_ACC: {
        int numreg = b0 % 2;
        return mnem[b0] + sep + "@R" + numreg + ", A";
      }
      case INDREG_ENDDADOS: {
        int numreg = b0 % 2;
        return mnem[b0] + sep + "@R" + numreg + ", " + SFRString(b1);
      }
      case INDREG_IMM: {
        int numreg = b0 % 2;
        return mnem[b0] + sep + "@R" + numreg + ", #" +
          Integer.toHexString(b1) + "H";
      }
      case INDREG_IMM_END8: {
        int numreg = b0 % 2;
        return mnem[b0] + sep + "@R" + numreg + ", #" +
          Integer.toHexString(b1) + "H, " + Integer.toHexString(b2) + "H";
      }
      case ENDDADOS_ENDDADOS: {
        return mnem[b0] + sep + SFRString(b2) + ", " + SFRString(b1);
      }
      case BIT_CARRY: {
        return mnem[b0] + sep + BitString(b1) + ", C";
      }
      case BIT_END8: {
        return mnem[b0] + sep + BitString(b1) + ", " +
          Integer.toHexString(b2) + "H";
      }
      case CARRY_BIT: {
        return mnem[b0] + sep + "C, " + BitString(b1);
      }
      case CARRY_COMPLBIT: {
        return mnem[b0] + sep + "C, /" + BitString(b1);
      }
      case ENDDADOS: {
        return mnem[b0] + sep + SFRString(b1);
      }
      case ENDDADOS_ACC: {
        return mnem[b0] + sep + SFRString(b1) + ", A";
      }
      case ENDDADOS_END8: {
        return mnem[b0] + sep + SFRString(b1) + ", " +
          Integer.toHexString(b2) + "H";
      }
      case ENDDADOS_IMM: {
        return mnem[b0] + sep + SFRString(b1) + ", #" +
          Integer.toHexString(b2) + "H";
      }
      case ENDDADOS_INDREG: {
        int numreg = b0 % 2;
        return mnem[b0] + sep + SFRString(b1) + ", @R" + numreg;
      }
      case ENDDADOS_REG: {
        int numreg = b0 % 8;
        return mnem[b0] + sep + SFRString(b1) + ", R" + numreg;
      }
      case DPTR_IMM: {
        return mnem[b0] + sep + "DPTR, #" + Integer.toHexString((b1<<8)+b2) + "H";
      }
      case INDDPTR_ACC: {
        return mnem[b0] + sep + "@DPTR, A";
      }
      default: {
        return "ERRO INTERNO (b0 =" + b0 + ", tipo[b0]=" + tipo[b0] + ") !!!!!";
      }
    }
  }

  public static void inicializa() {

    // 0X00 - NOP
    mnem[0x00]    = "NOP";
    tipo[0x00]    = SEMPAR;
    nbytes[0x00]  = (byte) 1;
    nciclos[0x00] = (byte) 12;

    // 0X01 - AJMP code addr
    mnem[0x01]    = "AJMP";
    tipo[0x01]    = END11;
    nbytes[0x01]  = (byte) 2;
    nciclos[0x01] = (byte) 24;

    // 0x02 - LJMP code addr
    mnem[0x02]    = "LJMP";
    tipo[0x02]    = END16;
    nbytes[0x02]  = (byte) 3;
    nciclos[0x02] = (byte) 24;

    // 0X03 - RR A
    mnem[0x03]    = "RR";
    tipo[0x03]    = ACC;
    nbytes[0x03]  = (byte) 1;
    nciclos[0x03] = (byte) 12;

    // 0X04 - INC A
    mnem[0x04]    = "INC";
    tipo[0x04]    = ACC;
    nbytes[0x04]  = (byte) 1;
    nciclos[0x04] = (byte) 12;

    // 0X05 - INC data addr
    mnem[0x05]    = "INC";
    tipo[0x05]    = ENDDADOS;
    nbytes[0x05]  = (byte) 2;
    nciclos[0x05] = (byte) 12;

    // 0x06 - INC @R0
    mnem[0x06]    = "INC";
    tipo[0x06]    = INDREG;
    nbytes[0x06]  = (byte) 1;
    nciclos[0x06] = (byte) 12;

    // 0x07 - INC @R1
    mnem[0x07]    = "INC";
    tipo[0x07]    = INDREG;
    nbytes[0x07]  = (byte) 1;
    nciclos[0x07] = (byte) 12;

    // 0x08 a 0x0f - INC Ri
    for(int i=0x08; i<0x10; i++) {
      mnem[i]    = "INC";
      tipo[i]    = REG;
      nbytes[i]  = (byte) 1;
      nciclos[i] = (byte) 12;
    }

    // 0x10 - JBC bit addr, code addr
    mnem[0x10]    = "JBC";
    tipo[0x10]    = BIT_END8;
    nbytes[0x10]  = (byte) 3;
    nciclos[0x10] = (byte) 24;

    // 0x11 - ACALL code addr
    mnem[0x11]    = "ACALL";
    tipo[0x11]    = END11;
    nbytes[0x11]  = (byte) 2;
    nciclos[0x11] = (byte) 24;

    // 0x12 - LCALL code addr
    mnem[0x12]    = "LCALL";
    tipo[0x12]    = END16;
    nbytes[0x12]  = (byte) 3;
    nciclos[0x12] = (byte) 24;

    // 0x13 - RRC A
    mnem[0x13]    = "RRC";
    tipo[0x13]    = ACC;
    nbytes[0x13]  = (byte) 1;
    nciclos[0x13] = (byte) 12;

    // 0x14 - DEC A
    mnem[0x14]    = "DEC";
    tipo[0x14]    = ACC;
    nbytes[0x14]  = (byte) 1;
    nciclos[0x14] = (byte) 12;

    // 0x15 - DEC data addr
    mnem[0x15]    = "DEC";
    tipo[0x15]    = ENDDADOS;
    nbytes[0x15]  = (byte) 2;
    nciclos[0x15] = (byte) 12;

    // 0x16 - DEC @R0
    mnem[0x16]    = "DEC";
    tipo[0x16]    = INDREG;
    nbytes[0x16]  = (byte) 1;
    nciclos[0x16] = (byte) 12;

    // 0x17 - DEC @R1
    mnem[0x17]    = "DEC";
    tipo[0x17]    = INDREG;
    nbytes[0x17]  = (byte) 1;
    nciclos[0x17] = (byte) 12;

    // 0x18 a 0x1f - DEC Ri
    for(int i=0x18; i<0x20; i++) {
      mnem[i]    = "DEC";
      tipo[i]    = REG;
      nbytes[i]  = (byte) 1;
      nciclos[i] = (byte) 12;
    }

    // 0x20 - JB bit addr, code addr
    mnem[0x20]    = "JB";
    tipo[0x20]    = BIT_END8;
    nbytes[0x20]  = (byte) 3;
    nciclos[0x20] = (byte) 24;

    // 0x21 - AJMP code addr
    mnem[0x21]    = "AJMP";
    tipo[0x21]    = END11;
    nbytes[0x21]  = (byte) 2;
    nciclos[0x21] = (byte) 24;

    // 0x22 - RET
    mnem[0x22]    = "RET";
    tipo[0x22]    = SEMPAR;
    nbytes[0x22]  = (byte) 1;
    nciclos[0x22] = (byte) 24;

    // 0x23 - RL A
    mnem[0x23]    = "RL";
    tipo[0x23]    = ACC;
    nbytes[0x23]  = (byte) 1;
    nciclos[0x23] = (byte) 12;

    // 0x24 - ADD A, #data
    mnem[0x24]    = "ADD";
    tipo[0x24]    = ACC_IMM;
    nbytes[0x24]  = (byte) 2;
    nciclos[0x24] = (byte) 12;

    // 0x25 - ADD A, data addr
    mnem[0x25]    = "ADD";
    tipo[0x25]    = ACC_ENDDADOS;
    nbytes[0x25]  = (byte) 2;
    nciclos[0x25] = (byte) 12;

    // 0x26 - ADD A, @R0
    mnem[0x26]    = "ADD";
    tipo[0x26]    = ACC_INDREG;
    nbytes[0x26]  = (byte) 1;
    nciclos[0x26] = (byte) 12;

    // 0x27 - ADD A, @R1
    mnem[0x27]    = "ADD";
    tipo[0x27]    = ACC_INDREG;
    nbytes[0x27]  = (byte) 1;
    nciclos[0x27] = (byte) 12;

    // 0x28 a 0x2f - ADD A, Ri
    for(int i=0x28; i<0x30; i++) {
      mnem[i]    = "ADD";
      tipo[i]    = ACC_REG;
      nbytes[i]  = (byte) 1;
      nciclos[i] = (byte) 12;
    }

    // 0x30 - JNB bit addr, code addr
    mnem[0x30]    = "JNB";
    tipo[0x30]    = BIT_END8;
    nbytes[0x30]  = (byte) 3;
    nciclos[0x30] = (byte) 24;

    // 0x31 - ACALL code addr
    mnem[0x31]    = "ACALL";
    tipo[0x31]    = END11;
    nbytes[0x31]  = (byte) 2;
    nciclos[0x31] = (byte) 24;

    // 0x32 - RETI
    mnem[0x32]    = "RETI";
    tipo[0x32]    = SEMPAR;
    nbytes[0x32]  = (byte) 1;
    nciclos[0x32] = (byte) 24;

    // 0x33 - RLC A
    mnem[0x33]    = "RLC";
    tipo[0x33]    = ACC;
    nbytes[0x33]  = (byte) 1;
    nciclos[0x33] = (byte) 12;

    // 0x34 - ADDC A, #data
    mnem[0x34]    = "ADDC";
    tipo[0x34]    = ACC_IMM;
    nbytes[0x34]  = (byte) 2;
    nciclos[0x34] = (byte) 12;

    // 0x35 - ADDC A, data addr
    mnem[0x35]    = "ADDC";
    tipo[0x35]    = ACC_ENDDADOS;
    nbytes[0x35]  = (byte) 2;
    nciclos[0x35] = (byte) 12;

    // 0x36 - ADDC A, @R0
    mnem[0x36]    = "ADDC";
    tipo[0x36]    = ACC_INDREG;
    nbytes[0x36]  = (byte) 1;
    nciclos[0x36] = (byte) 12;

    // 0x37 - ADDC A, @R1
    mnem[0x37]    = "ADDC";
    tipo[0x37]    = ACC_INDREG;
    nbytes[0x37]  = (byte) 1;
    nciclos[0x37] = (byte) 12;

    // 0x38 a 0x3f - ADDC A, Ri
    for(int i=0x38; i<0x40; i++) {
      mnem[i]    = "ADDC";
      tipo[i]    = ACC_REG;
      nbytes[i]  = (byte) 1;
      nciclos[i] = (byte) 12;
    }

    // 0x40 - JC code addr
    mnem[0x40]    = "JC";
    tipo[0x40]    = END8;
    nbytes[0x40]  = (byte) 2;
    nciclos[0x40] = (byte) 24;

    // 0x41 - AJMP code addr
    mnem[0x41]    = "AJMP";
    tipo[0x41]    = END11;
    nbytes[0x41]  = (byte) 2;
    nciclos[0x41] = (byte) 24;

    // 0x42 - ORL data addr, A
    mnem[0x42]    = "ORL";
    tipo[0x42]    = ENDDADOS_ACC;
    nbytes[0x42]  = (byte) 2;
    nciclos[0x42] = (byte) 12;

    // 0x43 - ORL data addr, #data
    mnem[0x43]    = "ORL";
    tipo[0x43]    = ENDDADOS_IMM;
    nbytes[0x43]  = (byte) 3;
    nciclos[0x43] = (byte) 24;

    // 0x44 - ORL A, #data
    mnem[0x44]    = "ORL";
    tipo[0x44]    = ACC_IMM;
    nbytes[0x44]  = (byte) 2;
    nciclos[0x44] = (byte) 12;

    // 0x45 - ORL A, data addr
    mnem[0x45]    = "ORL";
    tipo[0x45]    = ACC_ENDDADOS;
    nbytes[0x45]  = (byte) 2;
    nciclos[0x45] = (byte) 12;

    // 0x46 - ORL A, @R0
    mnem[0x46]    = "ORL";
    tipo[0x46]    = ACC_INDREG;
    nbytes[0x46]  = (byte) 1;
    nciclos[0x46] = (byte) 12;

    // 0x47 - ORL A, @R1
    mnem[0x47]    = "ORL";
    tipo[0x47]    = ACC_INDREG;
    nbytes[0x47]  = (byte) 1;
    nciclos[0x47] = (byte) 12;

    // 0x48 a 0x4f - ORL A, Ri
    for(int i=0x48; i<0x50; i++) {
      mnem[i]    = "ORL";
      tipo[i]    = ACC_REG;
      nbytes[i]  = (byte) 1;
      nciclos[i] = (byte) 12;
    }

    // 0x50 - JNC code addr
    mnem[0x50]    = "JNC";
    tipo[0x50]    = END8;
    nbytes[0x50]  = (byte) 2;
    nciclos[0x50] = (byte) 24;

    // 0x51 - ACALL code addr
    mnem[0x51]    = "ACALL";
    tipo[0x51]    = END11;
    nbytes[0x51]  = (byte) 2;
    nciclos[0x51] = (byte) 24;

    // 0x52 - ANL data addr, A
    mnem[0x52]    = "ANL";
    tipo[0x52]    = ENDDADOS_ACC;
    nbytes[0x52]  = (byte) 2;
    nciclos[0x52] = (byte) 12;

    // 0x53 - ANL data addr, #data
    mnem[0x53]    = "ANL";
    tipo[0x53]    = ENDDADOS_IMM;
    nbytes[0x53]  = (byte) 3;
    nciclos[0x53] = (byte) 24;

    // 0x54 - ANL A, #data
    mnem[0x54]    = "ANL";
    tipo[0x54]    = ACC_IMM;
    nbytes[0x54]  = (byte) 2;
    nciclos[0x54] = (byte) 12;

    // 0x55 - ANL A, data addr
    mnem[0x55]    = "ANL";
    tipo[0x55]    = ACC_ENDDADOS;
    nbytes[0x55]  = (byte) 2;
    nciclos[0x55] = (byte) 12;

    // 0x56 - ANL A, @R0
    mnem[0x56]    = "ANL";
    tipo[0x56]    = ACC_REG;
    nbytes[0x56]  = (byte) 1;
    nciclos[0x56] = (byte) 12;

    // 0x57 - ANL A, @R1
    mnem[0x57]    = "ANL";
    tipo[0x57]    = ACC_REG;
    nbytes[0x57]  = (byte) 1;
    nciclos[0x57] = (byte) 12;

    // 0x58 a 0x5f - ANL A, Ri
    for(int i=0x58; i<0x60; i++) {
      mnem[i]    = "ANL";
      tipo[i]    = ACC_REG;
      nbytes[i]  = (byte) 1;
      nciclos[i] = (byte) 12;
    }

    // 0x60 - JZ code addr
    mnem[0x60]    = "JZ";
    tipo[0x60]    = END8;
    nbytes[0x60]  = (byte) 2;
    nciclos[0x60] = (byte) 24;

    // 0x61 - AJMP code addr
    mnem[0x61]    = "AJMP";
    tipo[0x61]    = END11;
    nbytes[0x61]  = (byte) 2;
    nciclos[0x61] = (byte) 24;

    // 0x62 - XRL data addr, A
    mnem[0x62]    = "XRL";
    tipo[0x62]    = ENDDADOS_ACC;
    nbytes[0x62]  = (byte) 2;
    nciclos[0x62] = (byte) 12;

    // 0x63 - XRL data addr, #data
    mnem[0x63]    = "XRL";
    tipo[0x63]    = ENDDADOS_IMM;
    nbytes[0x63]  = (byte) 3;
    nciclos[0x63] = (byte) 24;

    // 0x64 - XRL A, #data
    mnem[0x64]    = "XRL";
    tipo[0x64]    = ACC_IMM;
    nbytes[0x64]  = (byte) 2;
    nciclos[0x64] = (byte) 12;

    // 0x65 - XRL A, data addr
    mnem[0x65]    = "XRL";
    tipo[0x65]    = ACC_ENDDADOS;
    nbytes[0x65]  = (byte) 2;
    nciclos[0x65] = (byte) 12;

    // 0x66 - XRL A, @R0
    mnem[0x66]    = "XRL";
    tipo[0x66]    = ACC_REG;
    nbytes[0x66]  = (byte) 1;
    nciclos[0x66] = (byte) 12;

    // 0x67 - XRL A, @R1
    mnem[0x67]    = "XRL";
    tipo[0x67]    = ACC_REG;
    nbytes[0x67]  = (byte) 1;
    nciclos[0x67] = (byte) 12;

    // 0x68 a 0x6f - XRL A, Ri
    for(int i=0x68; i<0x70; i++) {
      mnem[i]    = "XRL";
      tipo[i]    = ACC_REG;
      nbytes[i]  = (byte) 1;
      nciclos[i] = (byte) 12;
    }

    // 0x70 - JNZ code addr
    mnem[0x70]    = "JNZ";
    tipo[0x70]    = END8;
    nbytes[0x70]  = (byte) 2;
    nciclos[0x70] = (byte) 24;

    // 0x71 - ACALL code addr
    mnem[0x71]    = "ACALL";
    tipo[0x71]    = END11;
    nbytes[0x71]  = (byte) 2;
    nciclos[0x71] = (byte) 24;

    // 0x72 - ORL C, bit addr
    mnem[0x72]    = "ORL";
    tipo[0x72]    = CARRY_BIT;
    nbytes[0x72]  = (byte) 2;
    nciclos[0x72] = (byte) 24;

    // 0x73 - JMP @A + DPTR
    mnem[0x73]    = "JMP";
    tipo[0x73]    = ACC_MAIS_DPTR;
    nbytes[0x73]  = (byte) 1;
    nciclos[0x73] = (byte) 24;

    // 0x74 - MOV A, #data
    mnem[0x74]    = "MOV";
    tipo[0x74]    = ACC_IMM;
    nbytes[0x74]  = (byte) 2;
    nciclos[0x74] = (byte) 12;

    // 0x75 - MOV data addr, #data
    mnem[0x75]    = "MOV";
    tipo[0x75]    = ENDDADOS_IMM;
    nbytes[0x75]  = (byte) 3;
    nciclos[0x75] = (byte) 24;

    // 0x76 - MOV @R0, #data
    mnem[0x76]    = "MOV";
    tipo[0x76]    = INDREG_IMM;
    nbytes[0x76]  = (byte) 2;
    nciclos[0x76] = (byte) 12;

    // 0x77 - MOV @R1, #data
    mnem[0x77]    = "MOV";
    tipo[0x77]    = INDREG_IMM;
    nbytes[0x77]  = (byte) 2;
    nciclos[0x77] = (byte) 12;

    // 0x78 a 0x7f - MOV Ri, #data
    for(int i=0x78; i<0x80; i++) {
      mnem[i]    = "MOV";
      tipo[i]    = REG_IMM;
      nbytes[i]  = (byte) 2;
      nciclos[i] = (byte) 12;
    }

    // 0x80 - SJMP code addr
    mnem[0x80]    = "SJMP";
    tipo[0x80]    = END8;
    nbytes[0x80]  = (byte) 2;
    nciclos[0x80] = (byte) 24;

    // 0x81 - AJMP code addr
    mnem[0x81]    = "AJMP";
    tipo[0x81]    = END11;
    nbytes[0x81]  = (byte) 2;
    nciclos[0x81] = (byte) 24;

    // 0x82 - ANL C, bit addr
    mnem[0x82]    = "ANL";
    tipo[0x82]    = CARRY_BIT;
    nbytes[0x82]  = (byte) 2;
    nciclos[0x82] = (byte) 24;

    // 0x83 - MOVC A, @A + PC
    mnem[0x83]    = "ANL";
    tipo[0x83]    = ACC_ACC_MAIS_PC;
    nbytes[0x83]  = (byte) 1;
    nciclos[0x83] = (byte) 24;

    // 0x84 - DIV AB
    mnem[0x84]    = "DIV";
    tipo[0x84]    = ACC_B;
    nbytes[0x84]  = (byte) 1;
    nciclos[0x84] = (byte) 48;

    // 0x85 - MOV data addr, data addr
    mnem[0x85]    = "MOV";
    tipo[0x85]    = ENDDADOS_ENDDADOS;
    nbytes[0x85]  = (byte) 3;
    nciclos[0x85] = (byte) 24;

    // 0x86 - MOV data addr, @R0
    mnem[0x86]    = "MOV";
    tipo[0x86]    = ENDDADOS_INDREG;
    nbytes[0x86]  = (byte) 2;
    nciclos[0x86] = (byte) 24;

    // 0x87 - MOV data addr, @R1
    mnem[0x87]    = "MOV";
    tipo[0x87]    = ENDDADOS_INDREG;
    nbytes[0x87]  = (byte) 2;
    nciclos[0x87] = (byte) 24;

    // 0x88 a 0x8f - MOV data addr, Ri
    for(int i=0x88; i<0x90; i++) {
      mnem[i]    = "MOV";
      tipo[i]    = ENDDADOS_REG;
      nbytes[i]  = (byte) 2;
      nciclos[i] = (byte) 24;
    }

    // 0x90 - MOV DPTR, #data
    mnem[0x90]    = "MOV";
    tipo[0x90]    = DPTR_IMM;
    nbytes[0x90]  = (byte) 3;
    nciclos[0x90] = (byte) 24;

    // 0x91 - ACALL code addr
    mnem[0x91]    = "ACALL";
    tipo[0x91]    = END11;
    nbytes[0x91]  = (byte) 2;
    nciclos[0x91] = (byte) 24;

    // 0x92 - MOV bit addr, C
    mnem[0x92]    = "MOV";
    tipo[0x92]    = BIT_CARRY;
    nbytes[0x92]  = (byte) 2;
    nciclos[0x92] = (byte) 24;

    // 0x93 - MOVC A,@A + DPTR
    mnem[0x93]    = "MOVC";
    tipo[0x93]    = ACC_ACC_MAIS_DPTR;
    nbytes[0x93]  = (byte) 1;
    nciclos[0x93] = (byte) 24;

    // 0x94 - SUBB A, #data
    mnem[0x94]    = "SUBB";
    tipo[0x94]    = ACC_IMM;
    nbytes[0x94]  = (byte) 2;
    nciclos[0x94] = (byte) 12;

    // 0x95 - SUBB A, data addr
    mnem[0x95]    = "SUBB";
    tipo[0x95]    = ACC_ENDDADOS;
    nbytes[0x95]  = (byte) 2;
    nciclos[0x95] = (byte) 12;

    // 0x96 - SUBB data addr, @R0
    mnem[0x96]    = "SUBB";
    tipo[0x96]    = ENDDADOS_INDREG;
    nbytes[0x96]  = (byte) 2;
    nciclos[0x96] = (byte) 12;

    // 0x97 - SUBB data addr, @R1
    mnem[0x97]    = "SUBB";
    tipo[0x97]    = ENDDADOS_INDREG;
    nbytes[0x97]  = (byte) 2;
    nciclos[0x97] = (byte) 12;

    // 0x98 a 0x9f - SUBB data addr, Ri
    for(int i=0x98; i<0xa0; i++) {
      mnem[i]    = "SUBB";
      tipo[i]    = ENDDADOS_REG;
      nbytes[i]  = (byte) 2;
      nciclos[i] = (byte) 12;
    }

    // 0xa0 - ORL C, /bit addr
    mnem[0xa0]    = "ORL";
    tipo[0xa0]    = CARRY_COMPLBIT;
    nbytes[0xa0]  = (byte) 2;
    nciclos[0xa0] = (byte) 24;

    // 0xa1 - AJMP code addr
    mnem[0xa1]    = "AJMP";
    tipo[0xa1]    = END11;
    nbytes[0xa1]  = (byte) 2;
    nciclos[0xa1] = (byte) 24;

    // 0xa2 - MOV C, bit addr
    mnem[0xa2]    = "MOV";
    tipo[0xa2]    = CARRY_BIT;
    nbytes[0xa2]  = (byte) 2;
    nciclos[0xa2] = (byte) 12;

    // 0xa3 - INC DPTR
    mnem[0xa3]    = "INC";
    tipo[0xa3]    = DPTR;
    nbytes[0xa3]  = (byte) 1;
    nciclos[0xa3] = (byte) 24;

    // 0xa4 - MUL AB
    mnem[0xa4]    = "MUL";
    tipo[0xa4]    = ACC_B;
    nbytes[0xa4]  = (byte) 1;
    nciclos[0xa4] = (byte) 48;

    // 0xa5 - reserved
    mnem[0xa5]    = "reserved";
    tipo[0xa5]    = RES;
    nbytes[0xa5]  = (byte) 1;
    nciclos[0xa5] = (byte) 12;

    // 0xa6 - MOV @R0, data addr
    mnem[0xa6]    = "MOV";
    tipo[0xa6]    = INDREG_ENDDADOS;
    nbytes[0xa6]  = (byte) 2;
    nciclos[0xa6] = (byte) 24;

    // 0xa7 - MOV @R1, data addr
    mnem[0xa7]    = "MOV";
    tipo[0xa7]    = INDREG_ENDDADOS;
    nbytes[0xa7]  = (byte) 2;
    nciclos[0xa7] = (byte) 24;

    // 0xa8 a 0xaf - MOV Ri, data addr
    for(int i=0xa8; i<0xb0; i++) {
      mnem[i]    = "MOV";
      tipo[i]    = REG_ENDDADOS;
      nbytes[i]  = (byte) 2;
      nciclos[i] = (byte) 24;
    }

    // 0xb0 - ANL C, /bit addr
    mnem[0xb0]    = "ANL";
    tipo[0xb0]    = CARRY_COMPLBIT;
    nbytes[0xb0]  = (byte) 2;
    nciclos[0xb0] = (byte) 12;

    // 0xb1 - ACALL code addr
    mnem[0xb1]    = "ACALL";
    tipo[0xb1]    = END11;
    nbytes[0xb1]  = (byte) 2;
    nciclos[0xb1] = (byte) 24;

    // 0xb2 - CPL bit addr
    mnem[0xb2]    = "CPL";
    tipo[0xb2]    = BIT;
    nbytes[0xb2]  = (byte) 2;
    nciclos[0xb2] = (byte) 12;

    // 0xb3 - CPL C
    mnem[0xb3]    = "CPL";
    tipo[0xb3]    = CARRY;
    nbytes[0xb3]  = (byte) 1;
    nciclos[0xb3] = (byte) 12;

    // 0xb4 - CJNE A, #data, code addr
    mnem[0xb4]    = "CJNE";
    tipo[0xb4]    = ACC_IMM_END8;
    nbytes[0xb4]  = (byte) 3;
    nciclos[0xb4] = (byte) 24;

    // 0xb5 - CJNE A, data addr, code addr
    mnem[0xb5]    = "CJNE";
    tipo[0xb5]    = ACC_ENDDADOS_END8;
    nbytes[0xb5]  = (byte) 3;
    nciclos[0xb5] = (byte) 24;

    // 0xb6 - CJNE @R0, #data, code addr
    mnem[0xb6]    = "CJNE";
    tipo[0xb6]    = INDREG_IMM_END8;
    nbytes[0xb6]  = (byte) 3;
    nciclos[0xb6] = (byte) 24;

    // 0xb7 - CJNE @R1, #data, code addr
    mnem[0xb7]    = "CJNE";
    tipo[0xb7]    = INDREG_IMM_END8;
    nbytes[0xb7]  = (byte) 3;
    nciclos[0xb7] = (byte) 24;

    // 0xb8 a 0xbf - CJNE Ri, #data, code addr
    for(int i=0xb8; i<0xc0; i++) {
      mnem[i]    = "CJNE";
      tipo[i]    = REG_IMM_END8;
      nbytes[i]  = (byte) 3;
      nciclos[i] = (byte) 24;
    }

    // 0xc0 - PUSH data addr
    mnem[0xc0]    = "PUSH";
    tipo[0xc0]    = ENDDADOS;
    nbytes[0xc0]  = (byte) 2;
    nciclos[0xc0] = (byte) 24;

    // 0xc1 - AJMP code addr
    mnem[0xc1]    = "AJMP";
    tipo[0xc1]    = END8;
    nbytes[0xc1]  = (byte) 2;
    nciclos[0xc1] = (byte) 24;

    // 0xc2 - CLR bit addr
    mnem[0xc2]    = "CLR";
    tipo[0xc2]    = BIT;
    nbytes[0xc2]  = (byte) 2;
    nciclos[0xc2] = (byte) 12;

    // 0xc3 - CLR C
    mnem[0xc3]    = "CLR";
    tipo[0xc3]    = CARRY;
    nbytes[0xc3]  = (byte) 1;
    nciclos[0xc3] = (byte) 12;

    // 0xc4 - SWAP A
    mnem[0xc4]    = "SWAP";
    tipo[0xc4]    = ACC;
    nbytes[0xc4]  = (byte) 1;
    nciclos[0xc4] = (byte) 12;

    // 0xc5 - XCH A, data addr
    mnem[0xc5]    = "XCH";
    tipo[0xc5]    = ACC_ENDDADOS;
    nbytes[0xc5]  = (byte) 2;
    nciclos[0xc5] = (byte) 12;

    // 0xc6 - XCH A, @R0
    mnem[0xc6]    = "XCH";
    tipo[0xc6]    = ACC_INDREG;
    nbytes[0xc6]  = (byte) 1;
    nciclos[0xc6] = (byte) 12;

    // 0xc7 - XCH A, @R1
    mnem[0xc7]    = "XCH";
    tipo[0xc7]    = ACC_INDREG;
    nbytes[0xc7]  = (byte) 1;
    nciclos[0xc7] = (byte) 12;

    // 0xc8 a 0xcf - XCH A, Ri
    for(int i=0xc8; i<0xd0; i++) {
      mnem[i]    = "XCH";
      tipo[i]    = ACC_REG;
      nbytes[i]  = (byte) 1;
      nciclos[i] = (byte) 12;
    }

    // 0xd0 - POP data addr
    mnem[0xd0]    = "POP";
    tipo[0xd0]    = ENDDADOS;
    nbytes[0xd0]  = (byte) 2;
    nciclos[0xd0] = (byte) 24;

    // 0xd1 - ACALL code addr
    mnem[0xd1]    = "ACALL";
    tipo[0xd1]    = END11;
    nbytes[0xd1]  = (byte) 2;
    nciclos[0xd1] = (byte) 24;

    // 0xd2 - SETB bit addr
    mnem[0xd2]    = "SETB";
    tipo[0xd2]    = BIT;
    nbytes[0xd2]  = (byte) 2;
    nciclos[0xd2] = (byte) 12;

    // 0xd3 - SETB C
    mnem[0xd3]    = "SETB";
    tipo[0xd3]    = CARRY;
    nbytes[0xd3]  = (byte) 1;
    nciclos[0xd3] = (byte) 12;

    // 0xd4 - DA A
    mnem[0xd4]    = "DA";
    tipo[0xd4]    = ACC;
    nbytes[0xd4]  = (byte) 1;
    nciclos[0xd4] = (byte) 12;

    // 0xd5 - DJNZ data addr, code addr
    mnem[0xd5]    = "DJNZ";
    tipo[0xd5]    = ENDDADOS_END8;
    nbytes[0xd5]  = (byte) 3;
    nciclos[0xd5] = (byte) 24;

    // 0xd6 - XCHD A, @R0
    mnem[0xd6]    = "XCHD";
    tipo[0xd6]    = ACC_INDREG;
    nbytes[0xd6]  = (byte) 1;
    nciclos[0xd6] = (byte) 12;

    // 0xd7 - XCHD A, @R1
    mnem[0xd7]    = "XCHD";
    tipo[0xd7]    = ACC_INDREG;
    nbytes[0xd7]  = (byte) 1;
    nciclos[0xd7] = (byte) 12;

    // 0xd8 a 0xdf - DJNZ Ri, code addr
    for(int i=0xd8; i<0xe0; i++) {
      mnem[i]    = "DJNZ";
      tipo[i]    = REG_END8;
      nbytes[i]  = (byte) 2;
      nciclos[i] = (byte) 24;
    }

    // 0xe0 - MOVX A, @DPTR
    mnem[0xe0]    = "MOVX";
    tipo[0xe0]    = ACC_INDDPTR;
    nbytes[0xe0]  = (byte) 1;
    nciclos[0xe0] = (byte) 24;

    // 0xe1 - AJMP code addr
    mnem[0xe1]    = "AJMP";
    tipo[0xe1]    = END11;
    nbytes[0xe1]  = (byte) 2;
    nciclos[0xe1] = (byte) 24;

    // 0xe2 - MOVX A, @R0
    mnem[0xe2]    = "MOVX";
    tipo[0xe2]    = ACC_INDREG;
    nbytes[0xe2]  = (byte) 1;
    nciclos[0xe2] = (byte) 24;

    // 0xe3 - MOVX A, @R1
    mnem[0xe3]    = "MOVX";
    tipo[0xe3]    = ACC_INDREG;
    nbytes[0xe3]  = (byte) 1;
    nciclos[0xe3] = (byte) 24;

    // 0xe4 - CLR A
    mnem[0xe4]    = "CLR";
    tipo[0xe4]    = ACC;
    nbytes[0xe4]  = (byte) 1;
    nciclos[0xe4] = (byte) 12;

    // 0xe5 - MOV A, data addr
    mnem[0xe5]    = "MOV";
    tipo[0xe5]    = ACC_ENDDADOS;
    nbytes[0xe5]  = (byte) 2;
    nciclos[0xe5] = (byte) 12;

    // 0xe6 - MOV A, @R0
    mnem[0xe6]    = "MOV";
    tipo[0xe6]    = ACC_INDREG;
    nbytes[0xe6]  = (byte) 1;
    nciclos[0xe6] = (byte) 12;

    // 0xe7 - MOV A, @R1
    mnem[0xe7]    = "MOV";
    tipo[0xe7]    = ACC_INDREG;
    nbytes[0xe7]  = (byte) 1;
    nciclos[0xe7] = (byte) 12;

    // 0xe8 a 0xef - MOV A, Ri
    for(int i=0xe8; i<0xf0; i++) {
      mnem[i]    = "MOV";
      tipo[i]    = ACC_REG;
      nbytes[i]  = (byte) 1;
      nciclos[i] = (byte) 12;
    }

    // 0xf0 - MOVX @DPTR, A
    mnem[0xf0]    = "MOVX";
    tipo[0xf0]    = INDDPTR_ACC;
    nbytes[0xf0]  = (byte) 1;
    nciclos[0xf0] = (byte) 24;

    // 0xf1 - ACALL code addr
    mnem[0xf1]    = "ACALL";
    tipo[0xf1]    = END11;
    nbytes[0xf1]  = (byte) 2;
    nciclos[0xf1] = (byte) 24;

    // 0xf2 - MOVX @R0, A
    mnem[0xf2]    = "MOVX";
    tipo[0xf2]    = INDREG_ACC;
    nbytes[0xf2]  = (byte) 1;
    nciclos[0xf2] = (byte) 24;

    // 0xf3 - MOVX @R1, A
    mnem[0xf3]    = "MOVX";
    tipo[0xf3]    = INDREG_ACC;
    nbytes[0xf3]  = (byte) 1;
    nciclos[0xf3] = (byte) 24;

    // 0xf4 - CPL A
    mnem[0xf4]    = "CPL";
    tipo[0xf4]    = ACC;
    nbytes[0xf4]  = (byte) 1;
    nciclos[0xf4] = (byte) 12;

    // 0xf5 - MOV data addr, A
    mnem[0xf5]    = "MOV";
    tipo[0xf5]    = ENDDADOS_ACC;
    nbytes[0xf5]  = (byte) 2;
    nciclos[0xf5] = (byte) 12;

    // 0xf6 - MOV @R0, A
    mnem[0xf6]    = "MOV";
    tipo[0xf6]    = INDREG_ACC;
    nbytes[0xf6]  = (byte) 1;
    nciclos[0xf6] = (byte) 12;

    // 0xf7 - MOV @R1, A
    mnem[0xf7]    = "MOV";
    tipo[0xf7]    = INDREG_ACC;
    nbytes[0xf7]  = (byte) 1;
    nciclos[0xf7] = (byte) 12;

    // 0xf8 a 0xff - MOV Ri, A
    for(int i=0xf8; i<0x100; i++) {
      mnem[i]    = "MOV";
      tipo[i]    = REG_ACC;
      nbytes[i]  = (byte) 1;
      nciclos[i] = (byte) 12;
    }

    pronto = true;
  }
}
