
//Title:        Emulador do Kit MC8051
//Version:
//Copyright:    Copyright (c) 1997
//Author:       Hermes F. Reis Neto / Jan L. de Lara / Kleber w. Tolezani
//Company:      EPUSP
//Description:


package MC8051;

import java.io.*;

public class Memoria {

  // constantes correspondentes aos endereos dos registradores especiais na
  // RAM interna
  public static final int P0   = 0x80;
  public static final int SP   = 0x81;
  public static final int DPL  = 0x82;
  public static final int DPH  = 0x83;
  public static final int PCON = 0x87;
  public static final int TCON = 0x88;
  public static final int TMOD = 0x89;
  public static final int TL0  = 0x8a;
  public static final int TL1  = 0x8b;
  public static final int TH0  = 0x8c;
  public static final int TH1  = 0x8d;
  public static final int P1   = 0x90;
  public static final int SCON = 0x98;
  public static final int SBUF = 0x99;
  public static final int P2   = 0xa0;
  public static final int IE   = 0xa8;
  public static final int P3   = 0xb0;
  public static final int IP   = 0xb8;
  public static final int PSW  = 0xd0;
  public static final int ACC  = 0xe0;
  public static final int B    = 0xf0;

  // constantes correspondentes aos endereos dos bits do registrador PSW
  public static final int PSW_CY = 0xd7;
  public static final int PSW_AC = 0xd6;
  public static final int PSW_OV = 0xd2;
  public static final int PSW_P  = 0xd0;

  // constantes correspondentes a endereos de bits do registrador TCON
  public static final int TCON_TF1 = 0x8f;
  public static final int TCON_TF0 = 0x8d;

  // constantes correspondentes a endereos de bits do registrador SCON
  public static final int SCON_RI = 0x98;
  public static final int SCON_TI = 0x99;

  // constantes correspondentes aos endereos das rotinas de tratamento de
  // interrupes
  public static final int INT_EXTI0 = 0x03;
  public static final int INT_TIMER0 = 0x0b;
  public static final int INT_EXTI1 = 0x13;
  public static final int INT_TIMER1 = 0x1b;
  public static final int INT_SERIAL = 0x23;

  // cdigos de erro da funo leArquivoHex()
  public static final int ERRO_HEX_CHKSUM = -1;
  public static final int ERRO_HEX_TIPOREG = -2;
  public static final int ERRO_HEX_FORMATO = -3;
  public static final int ERRO_HEX_LEITURA = -4;

  // indicadores de alterao em IE,IP,TCON,TMOD e SCON e execuo de RETI
  public static boolean IE_alterado = false;
  public static boolean IP_alterado = false;
  public static boolean TCON_alterado = false;
  public static boolean TMOD_alterado = false;
  public static boolean SCON_alterado = false;
  public static boolean PCON_alterado = false;
  public static boolean SBUF_alterado = false;
  public static boolean RETI_executada = false;
  public static boolean TIMER1_overflow = false;

  // variveis
  public static GerenciadorES es;
  public static int bancoreg;
  public static int PC;
  public static int INTatual_n0 = 0;
  public static int INTatual_n1 = 0;
  public static short[] memint = new short[0x100];
  public static short[] memext = new short[0x10000];
  public static boolean[] breakpoint = new boolean[0x10000];
  public static short SBUF_tx;

  public static void inicializa(boolean apagar_mem) {
    IE_alterado = false;
    IP_alterado = false;
    TCON_alterado = false;
    TMOD_alterado = false;
    SCON_alterado = false;
    PCON_alterado = false;
    SBUF_alterado = false;
    RETI_executada = false;
    TIMER1_overflow = false;
    bancoreg = 0;
    PC = 0;
    INTatual_n0 = 0;
    INTatual_n1 = 0;

    for(int i=0; i<0x100; i++)
      memint[i] = 0;
    memint[SP] = 0x07;
    memint[SCON] = 0x03;
    SBUF_tx = 0;

    if(apagar_mem) {
      for(int i=0; i<0x10000; i++) {
        memext[i] = 0;
        breakpoint[i] = false;
      }
      // EXTI0 -> 0Xfff0
      memext[0x03] = 0x02;
      memext[0x04] = 0xff;
      memext[0x05] = 0xf0;
      // TIMER0 -> 0Xfff3
      memext[0x0b] = 0x02;
      memext[0x0c] = 0xff;
      memext[0x0d] = 0xf3;
      // EXTI1  -> 0Xfff6
      memext[0x13] = 0x02;
      memext[0x14] = 0xff;
      memext[0x15] = 0xf6;
      // TIMER1 -> 0Xfff9
      memext[0x1b] = 0x02;
      memext[0x1c] = 0xff;
      memext[0x1d] = 0xf9;
      // SERIAL -> 0Xfffc
      memext[0x23] = 0x02;
      memext[0x24] = 0xff;
      memext[0x25] = 0xfc;
    }
  }

  // carrega os dados de um arquivo formato Intel HEX para a memria.
  // formato da linha:
  // :<numbytes><endereco><tiporeg><dados_da_linha><chksum>
  // 1    2         4         2       2*numbytes      2
  public static int leArquivoHEX(String nomeArq) {
    BufferedReader entr; // Stream de leitura

    // abre arquivo para leitura
    try {
      entr = new BufferedReader(new FileReader(nomeArq));
    } catch(IOException e) {
      System.err.println("leArquivoHEX(): excecao: " + e);
      return ERRO_HEX_LEITURA;
    }

    // l primeira linha do arquivo
    String linha = null;
    try {
      linha = entr.readLine();
    } catch(IOException e) {
      try{ entr.close(); } catch(IOException e1) {}
      System.err.println("leArquivoHEX(): excecao: " + e);
      return ERRO_HEX_LEITURA;
    }

    // processa linha e repete at encontrar fim do arquivo ou registro tipo 01.
    // calcula checksum como sendo o complemento de 2 da soma de todos os bytes
    // lidos (cada grupo de 2 caracteres corresponde a um byte em hexa) 
    // exceo do ltimo, que  o valor do checksum da linha.
    while(linha != null) {

      // verifica se linha inicia por : - se no for retorna erro
      if(linha.charAt(0) != ':') {
        try { entr.close(); } catch (IOException e) {}
        return ERRO_HEX_FORMATO;
      }

      // zera checksum calculado
      int chksum_calc = 0;
      try {

        // l campo <numbytes> - 2 caracteres
        int numbytes = Integer.parseInt(linha.substring(1,3),16);
        chksum_calc += numbytes;

        // l parte mais significativa do campo <endereco> - 2 caracteres
        int endereco = Integer.parseInt(linha.substring(3,5),16);
        chksum_calc += endereco;
        endereco <<= 8;

        // l parte menos significativa do campo <endereco> - 2 caracteres
        endereco += Integer.parseInt(linha.substring(5,7),16);
        chksum_calc += endereco;

        // l campo <tiporeg> - 2 caracteres
        int tiporeg = Integer.parseInt(linha.substring(7,9),16);
        chksum_calc += tiporeg;

        int chksum_arq = 0;
        // processa registro tipo 00 - dados
        if(tiporeg == 0) {
          if(numbytes > 0) {
            int pos = 9;
            while(numbytes > 0) {
              int dado = Integer.parseInt(linha.substring(pos,pos+2),16);
              chksum_calc += dado;
              memext[endereco] = (short)dado;
              endereco++;
              pos+=2;
              numbytes--;
            }
            chksum_arq = Integer.parseInt(linha.substring(pos),16);
            try {
              linha = entr.readLine();
            } catch(IOException e1) {
              System.err.println("leArquivoHEX(): excecao: " + e1);
            }
          }
          else {
            chksum_arq = Integer.parseInt(linha.substring(9),16);
            try {
              linha = entr.readLine();
            } catch(IOException e1) {
              System.err.println("leArquivoHEX(): excecao: " + e1);
            }
          }
        }
        // processa registro tipo 1 - fim de arquivo
        else if(tiporeg == 1) {
          chksum_arq = Integer.parseInt(linha.substring(9),16);
          linha = null;
        }
        // retorna erro para outros tipos de registro
        else {
          try { entr.close(); } catch (IOException e1) {}
          System.err.println(
            "leArquivoHEX(): tipo de registro invlido ou nao suportado: " +
            tiporeg
          );
          return ERRO_HEX_TIPOREG;
        }

        // calcula complemento de 2, com apenas 1 byte, do checksum
        chksum_calc = ((~chksum_calc)+1) & 0xff;

        // se o checksum no confere, retorna erro
        if(chksum_calc != chksum_arq) {
          try { entr.close(); } catch(IOException e1) {}
          return ERRO_HEX_CHKSUM;
        }

      } catch(NumberFormatException e) {
        try { entr.close(); } catch (IOException e1) {}
        System.err.println("leArquivoHEX(): excecao: " + e);
        return ERRO_HEX_FORMATO;

      } catch(StringIndexOutOfBoundsException e) {
        try { entr.close(); } catch (IOException e1) {}
        System.err.println("leArquivoHEX(): excecao: " + e);
        return ERRO_HEX_FORMATO;
      }
    }

    // fecha arquivo
    try { entr.close(); } catch(IOException e) {}
    return 0;
  }

  public static void imprimeEstado() {
    System.out.println(
      "PC=0x" + Integer.toHexString(PC) + "\t" +
      Instr.ASMString(memext[PC], memext[(PC+1)&0xffff],memext[(PC+2)&0xffff])
    );

    System.out.println(
      "SP=0x" + Integer.toHexString(memint[SP]) +
      "\t(SP)=0x" + Integer.toHexString(memint[memint[SP]])
    );

    int PSW = memint[Memoria.PSW];
    System.out.print("CY=" + ((PSW&0x80)>>7));
    System.out.print("\tAC=" + ((PSW&0x40)>>6));
    System.out.print("\tOV=" + ((PSW&0x04)>>2));
    System.out.print("\tP=" + (PSW&0x01));
    System.out.println("\tPSW=0x" + Integer.toHexString(PSW));

    System.out.println();
    System.out.println(
      "IE=0x" + Integer.toHexString(memint[IE]) +
      "\tIP=0x" + Integer.toHexString(memint[IP])
    );


    System.out.println();
    System.out.println(
      "A=0x" + Integer.toHexString(memint[ACC]) +
      "\tB=0x" + Integer.toHexString(memint[B]) +
      "\tDPTR=0x" + Integer.toHexString((memint[DPH]<<8)+memint[DPL])
    );

    for(int r=0; r<8; r++) {
      System.out.print(
        "R" + r + "=0x" + Integer.toHexString(memint[endReg(r)])
      );

      if(r < 7)
        System.out.print("\t");
      else
        System.out.println();
    }

    int TCON = memint[Memoria.TCON];
    int TMOD = memint[Memoria.TMOD];
    if( (TCON&0x10) > 0) {
      System.out.println();
      System.out.println("** Timer 0:");
      System.out.println(
        "\tGATE=" + ((TMOD&0x08)>>3) +
        "\tC/T=" + ((TMOD&0x04)>>2) +
        "\tmodo=" + (TMOD&0x03)
//        "\tTR0=" + ((TCON&0x10)>>4)
      );
      System.out.println(
        "\tTF0=" + ((TCON&0x20)>>5) +
        "\tTH0=0x" + Integer.toHexString(memint[TH0]) +
        "\tTL0=0x" + Integer.toHexString(memint[TL0])
      );
    }

    if( (TCON&0x40) > 0) {
      System.out.println();
      System.out.println("** Timer 1:");
      System.out.println(
        "\tGATE=" + ((TMOD&0x80)>>7) +
        "\tC/T=" + ((TMOD&0x40)>>6) +
        "\tmodo=" + ((TMOD&0x30)>>4)
//        "\tTR1=" + ((TCON&0x40)>>6)
      );
      System.out.println(
        "\tTF1=" + ((TCON&0x80)>>7) +
        "\tTH1=0x" + Integer.toHexString(memint[TH1]) +
        "\tTL1=0x" + Integer.toHexString(memint[TL1])
      );
    }
  }

  public static int leRAMInt(int endereco) {
    if(endereco == P0) {
      return 0xff;
    }
    if(endereco == P1) {
      return es.leTeclado();
    }
    return memint[endereco];
  }

  public static int leRAMExt(int endereco) {
    if(endereco < 0x8000)
      return (es.leDado(endereco));
    else
      return memext[endereco];
  }

  public static int leROM(int endereco) {
    return memext[endereco];
  }

  public static int endReg(int numreg) {
    bancoreg = (memint[PSW] & 0x18) >> 3;

    return numreg + 8 * bancoreg;
  }

  public static void gravaRAMInt(int endereco, short valor) {

    if(endereco == SBUF) {
//      System.out.println("Memoria: SBUF alterado");
      SBUF_tx = valor;
      SBUF_alterado = true;
      return;
    }

    memint[endereco] = valor;

    switch(endereco) {
      case IE:   {   IE_alterado = true; break; }
      case IP:   {   IP_alterado = true; break; }
      case TCON: { TCON_alterado = true; break; }
      case TMOD: { TMOD_alterado = true; break; }
      case SCON: { SCON_alterado = true; break; }
      case PCON: { PCON_alterado = true; break; }
    }
  }

  public static void gravaRAMExt(int endereco, short valor) {
    if(endereco < 0x8000)
      es.escreveDado(endereco, valor);
    else
      memext[endereco] = valor;
  }

  // Efetua a leitura de um bit, cujo endereo foi fornecido, e retorna TRUE se
  // o mesmo estiver setado; FALSE caso contrrio.
  // OBS: Se o bit for de uma porta de E/S (P0 a P3)  efeituada uma leitura de
  // dados na a porta.
  public static boolean bitSetado(int endbit) {
    int posbit = endbit % 8;

    int valreg;
    if(endbit < 0x80)
      valreg = leRAMInt(endbit / 8 + 0x20);
    else
      valreg = leRAMInt(endbit - posbit);

    int mascara = 1 << posbit;

    return ( (valreg & mascara) > 0);
  }

  // Efetua a leitura de um bit, cujo endereo foi fornecido, e retorna TRUE se
  // o mesmo estiver setado; FALSE caso contrrio.
  // Utilizada nas instrues "JBC bit addr, code addr"  e "CPL bit addr"
  // OBS: Se o bit for de uma porta de E/S (P0 a P3)  utilizado o ltimo valor
  // lido da porta (valor do latch interno e no dos pinos).
  public static boolean bitSetado2(int endbit) {
    int posbit = endbit % 8;

    int valreg;
    if(endbit < 0x80)
      valreg = memint[endbit / 8 + 0x20];
    else
      valreg = memint[endbit - posbit];

    int mascara = 1 << posbit;

    return ( (valreg & mascara) > 0);
  }

  // Altera o valor do bit, cujo endereo foi fornecido, para '1', caso "setar"
  // seja TRUE, ou para '0', caso "setar" seja FALSE.
  // OBS: Se o bit for de uma porta de E/S (P0 a P3)  utilizado o ltimo valor
  // lido da porta (valor do latch interno e no dos pinos).
  public static void alteraBit(int endbit, boolean setar) {

    int posbit = endbit % 8;

    int endreg;
    if(endbit < 0x80)
      endreg = endbit / 8 + 0x20;
    else
      endreg = endbit - posbit;

    short mascara = (short)(1 << posbit);
    short valreg;
    if( setar )
      valreg = (short) (memint[endreg] | mascara);
    else
      valreg = (short) (memint[endreg] & ~mascara);

    gravaRAMInt(endreg, valreg);
  }
}
