
//Title:        Emulador para o Kit MC8051
//Version:
//Copyright:    Copyright (c) 1997
//Author:       Hermes F. Reis Neto / Jan L. de Lara / Kleber W. Tolezani
//Company:      Escola Politcnica da Universidade de So Paulo
//Description:


package MC8051;

import java.io.*;

public class Serial {
  public int modo;  // modo de operao da serial (0 a 3)
  public int SCON;  // valor do registrador SCON
  public int dois_a_SMOD; // valor de 2 elevado a SMOD
  public boolean REN_setado; // true se bit REN de SCON estiver setado
  public int nciclos_acum; // contador de ciclos
  public int nciclos_bit; // nmero de ciclos correspondente a 1 bit TX ou RX
  public int nbit_tx; // contador de bits de transmisso
  public int nbit_rx; // contador de bits de recepo
  public int max_nbit; // limite do contador de bits
  public boolean transmitindo; // true se transmisso em andamento
  public boolean recebendo; // true se recepo em andamento
  public DataInputStream entr; // Stream de dados de entrada

  public Serial() {
    reset();
    entr = new DataInputStream(System.in);
  }

  // verifica se h dados pendentes para leitura
  public boolean dadosPendentes() {
    boolean res = false;
    try {
      res = (entr.available() > 0);
    } catch (IOException e) {
      System.out.println("\n\nERRO: Excecao: " + e);
      System.exit(1);
    }
    return res;
  }

  // reinicializa serial
  public void reset() {
    nciclos_acum = 0;
    configura();
  }

  // reconfigura modo de operao
  public void configura() {
    int div=0;
    SCON = Memoria.memint[Memoria.SCON];
    if((Memoria.memint[Memoria.PCON] & 0x80) > 0)
      div=2;
    else
      div=1;

    modo = SCON >> 6;
    REN_setado = (SCON & 0x10) > 0;

    switch(modo) {
      case 0: { nciclos_bit = 12; max_nbit = 8; break; }
      case 1: { nciclos_bit = 32/div; max_nbit = 10; break; }
      case 2: { nciclos_bit = 64/div; max_nbit = 11; break; }
      case 3: { nciclos_bit = 32/div; max_nbit = 11; break;}
    }
/*
    System.out.print("Serial: configurando: ");
    System.out.print("modo=" + modo);
    if(REN_setado)
      System.out.print(", REN=1");
    else
      System.out.print(", REN=0");
    System.out.println();
*/
  }

  // inicia transmisso
  public void iniciaTX() {
//    System.out.println("Serial: iniciando TX");
    nbit_tx = 0;
    transmitindo = true;
  }

  // inicia recepo
  public void iniciaRX() {
//    System.out.println("Serial: iniciando RX");
    if(REN_setado) {
      nbit_rx = 0;
      recebendo = true;
    }
  }

  // imprime dado transitido no console
  public void imprimeSBUF() {
    System.out.print((char)Memoria.SBUF_tx);
  }

  // le dado do console
  public void carregaSBUF() {
    try {
      Memoria.memint[Memoria.SBUF] = (short)entr.read();
    } catch (IOException e) {
      System.out.println("\n\nERRO: excecao: " + e);
      System.exit(1);
    }
  }

  // transmite/recebe um bit para cada clock
  public void recebeClock(int nclocks) {
//    System.out.println("Serial: recebi " + nclocks + " clocks");
    if(transmitindo) {
      nbit_tx += nclocks;

      if(nbit_tx >= max_nbit) {
        imprimeSBUF();
        Memoria.memint[Memoria.SCON] =
          (short)(Memoria.memint[Memoria.SCON] | 0x02);
        transmitindo = false;
      }
    }

    if(recebendo) {
      nbit_rx += nclocks;

      if(nbit_rx >= max_nbit) {
        carregaSBUF();
        Memoria.memint[Memoria.SCON] =
          (short)(Memoria.memint[Memoria.SCON] | 0x01);
        recebendo = false;
      }
    }
  }

  // adiciona <nciclos> de relgio ao tempo de simulao
  public void computaCiclos(int nciclos) {
    int nclocks=0;

    if(Memoria.SCON_alterado || Memoria.PCON_alterado) {
      configura();
    }

    if(Memoria.SBUF_alterado) {
      iniciaTX();
    }
    if(!recebendo && dadosPendentes()) {
      iniciaRX();
    }

    switch(modo) {
      case 0: { nciclos_acum+= nciclos; break; }
      case 2: { nciclos_acum += nciclos; break; }
      default: { if(Memoria.TIMER1_overflow) nciclos_acum++; }
    }

    nclocks = nciclos_acum / nciclos_bit;
    if(nclocks > 0)
      recebeClock(nclocks);

    if(nciclos_acum >= nciclos_bit)
      nciclos_acum %= nciclos_bit;
  }
}