
//Title:        Emulador para o Kit MC8051
//Version:
//Copyright:    Copyright (c) 1997
//Author:       Hermes F. Reis Neto / Jan L. de Lara / Kleber W. Tolezani
//Company:      Escola Politcnica da Universidade de So Paulo
//Description:


package MC8051;

public class Timers {

  public boolean gateT0setado, gateT1setado;
  public boolean ctT0setado, ctT1setado;
  public int modoT0, modoT1;
  public boolean TR0setado, TR1setado;

  public Timers() {
    reset();
  }

  public void reset() {
    gateT1setado = false;
    gateT0setado = false;
    ctT1setado = false;
    ctT0setado = false;
    modoT1 = 0;
    modoT0 = 0;

    TR1setado = false;
    TR0setado = false;
  }

  // reconfigura modo de operao
  public void configura() {
    int TMOD = Memoria.memint[Memoria.TMOD];
    int TCON = Memoria.memint[Memoria.TCON];

    gateT1setado = (TMOD&0x80) > 0;
    gateT0setado = (TMOD&0x08) > 0;
    ctT1setado = (TMOD&0x40) > 0;
    ctT0setado = (TMOD&0x04) > 0;
    modoT1 = (TMOD&0x30)>>4;
    modoT0 = (TMOD&0x03);

    TR1setado = (TCON&0x40) > 0;
    TR0setado = (TCON&0x10) > 0;
  }

  // incrementa contador <numtimer> <nclocks> vezes
  // se modo3 for true, numtimer=1 refere-se a TH0
  public void conta(int numtimer, int nclocks, boolean modo3) {
    int endTH, endTL, endTF;
    int TH,TL,T;
    int modo;
    boolean setarTF = false;

    if(modo3) {
      if(modoT0 != 3) {
        System.out.println("ERRO em Timers.conta(): modo3 est setado mas modo"+
          " do Timer0 eh " + modoT0);
        return;
      }

      modo = 3;
      endTH=-1;
      if(numtimer == 0) {
        endTL = Memoria.TL0;
        endTF = Memoria.TCON_TF0;
      }
      else {
        endTL = Memoria.TH0;
        endTF = Memoria.TCON_TF1;
      }
    }
    else if(numtimer == 0) {
      endTH = Memoria.TH0;
      endTL = Memoria.TL0;
      endTF = Memoria.TCON_TF0;
      modo = modoT0;
    }
    else {
      if(modoT1 == 3)
        return;
      endTH = Memoria.TH1;
      endTL = Memoria.TL1;
      endTF = Memoria.TCON_TF1;
      modo = modoT1;
    }

    TL = Memoria.memint[endTL];
    if(!modo3)
      TH = Memoria.memint[endTH];
    else
      TH = -1;

    switch(modo) {
      // MODO 0: CONTADOR DE 13 BITS: TL FUNCIONA COMO DIVISOR POR 32 (5 BITS)
      case 0: {
        T = (TH<<5) + (TL&0x1f);

        T += nclocks;
        if(T > 0x1fff) {
          T &= 0x1fff;
          setarTF = true;
        }

        TH = T>>5;
        TL = T&0xff;

        break;
      }
      // MODO 1: CONTADOR DE 16 BITS: TH CONCATENADO COM TL
      case 1: {
        T = (TH<<8) + TL;

        T += nclocks;
        if(T > 0xffff) {
          T &= 0xffff;
          setarTF = true;
        }

        TH = T>>8;
        TL = (T&0xff);

        break;
      }
      // MODO 2: CONTADOR DE 8 BITS: TL C/ PRESET POR TH
      case 2: {
        while(nclocks > 0) {
          if(TL == 0xff) {
            TL = TH;
            setarTF = true;
          }
          else {
            TL++;
          }
          nclocks--;
        }
        break;
      }
      // MODO 3: 2 CONTADORES INDEPENDENTES DE 8 BITS: TL0 (0) E TH0 (1) - TH0 
      // CONTROLADO PELOS BITS DE CONTROLE DO TIMER1.
      default: {
        TL += nclocks;
        if(TL > 0xff) {
          TL &= 0xff;
          setarTF = true;
        }
      }
    } // FIM switch()

    Memoria.memint[endTL] = (short)TL;
    if(setarTF) {
      if(numtimer == 1)
        Memoria.TIMER1_overflow = true;
      Memoria.alteraBit(endTF, true);
    }
    if(!modo3)
      Memoria.memint[endTH] = (short)TH;
  }

  // adiciona <nciclos> de relgio ao tempo de simulao
  public void computaCiclos(int nciclos) {
    if(Memoria.TCON_alterado || Memoria.TMOD_alterado) {
      configura();
    }

    if(TR0setado && !ctT0setado && !gateT0setado) {
      conta(0, nciclos/12, false);
    }

    if(modoT0 == 3 && TR1setado && !ctT1setado && !gateT1setado) {
      conta(1, nciclos/12, true);
    }

    if(TR1setado && !ctT1setado && !gateT1setado) {
      conta(1, nciclos/12, false);
    }
  }
}
